@@ -4,7 +4,9 @@ __all__ = [
     'NotebookExecutorBase',
     'InMemoryQueryExecutorBase',
     'FeatureComputeBase',
+    'FeatureExtractorBase',
     'FeaturesetExtractorBase',
+    'note',
     'MetricsBase'
 ]
 
