@@ -148,6 +148,7 @@ class InMemoryQueryExecutorBase(Compute):
         targets = "|".join(targets)
 
         # Now construct the args dynamically
+        remaining = self.supported_extra_args
         return [
             {
                 "name": "targets",
@@ -173,7 +174,7 @@ class InMemoryQueryExecutorBase(Compute):
                 "default": get_today(),
                 "required": True
             }
-        ]
+        ] + remaining
 
     def update_frame(self, name, engine, sql, df, dependencies=[]):
         """
@@ -455,6 +456,15 @@ class InMemoryQueryExecutorBase(Compute):
                     msg += "Columns: {}\n".format(", ".join(df.columns))
                     msg += "Dtypes: " + df.dtypes.to_string() + '\n'
 
+                    if len(df) == 0:
+                        # no data returned...
+                        logger.warning(f"Completed {name} {params['start_date']} No data",
+                                     extra={
+                                         'transform': self.name,
+                                         'data': msg
+                                 })
+                        continue
+
                     # First gather a map of segments
                     filemap = {}
                     if segmentcol is None:
