@@ -84,6 +84,17 @@ class MetricsBase(Compute):
     def instantiable(cls):
         return False
 
+    def get_printable_db_uri(self, engine):
+        """
+        pretty print the URL
+        """
+        username = engine.url.username
+        host          = engine.url.host
+        database   = engine.url.database
+        drivername= engine.url.get_driver_name()
+
+        return f"{drivername}:///{host}/{database}/"
+
     def get_db_uri(self, source):
         """
         Return database URI for a source
@@ -123,7 +134,7 @@ class MetricsBase(Compute):
                 continue
             if p.endswith(".json"):
                 profile = json.load(open(p))
-            else:
+            elif p.endswith(".yaml"):
                 profile = yaml.load(open(p))
 
         if profile is None:
@@ -157,6 +168,37 @@ class MetricsBase(Compute):
             query = quer(source)
         return query
 
+    def update_frame(self, source, df, lineage=None):
+
+        if isinstance(source, str):
+            name = source
+            description = ""
+        else:
+            name = source['name']
+            description = source.get('description','')
+
+        if self.state.has_frame(name):
+            return
+
+        params = self.get_column_params(name, df)
+        if lineage is not None:
+            if isinstance(lineage, dict):
+                params.append(lineage)
+            else:
+                params.extend(lineage)
+
+        detail = {
+            'name': name,
+            'df': df,
+            'frametype': 'pandas',
+            'description': description,
+            'params': params,
+            'transform': self.name,
+            'history': []
+        }
+
+        self.state.update_frame(name, detail)
+
     def read_db_source(self, source):
 
         # Get the SQLAlchemy URI
@@ -171,9 +213,24 @@ class MetricsBase(Compute):
         # Run the query
         df = pd.read_sql_query(satext(query), con=engine)
 
+        # Now the input load...
+        lineage = {
+            "type": "lineage",
+            "transform": self.name,
+            "dependencies": [
+                {
+                    "type": "database",
+                    "nature": "input",
+                    "objects": [self.get_printable_db_uri(uri)]
+                },
+            ]
+        }
+
+        self.update_frame(source, df, lineage)
+
         return df
 
-    def get_datasets(self, profile):
+    def get_datasets(self, profile, specs):
         """
         Load the datasets specified by the profile
         """
@@ -195,6 +252,11 @@ class MetricsBase(Compute):
             handlers = {}
 
 
+        required_sources = []
+        for s in specs:
+            required_sources.extend(s['sources'])
+        required_sources = list(set(required_sources))
+
         # Now no about constructucting the datasets
         datasets = {}
 
@@ -203,38 +265,59 @@ class MetricsBase(Compute):
 
             nature = source.get('nature', 'db')
             name = source['name']
+
+            if name not in required_sources:
+                continue
+
             pipeline = source.get('pipeline', None)
             generate = source.get('generate', None)
 
             # Only db is used for now...
-            if nature == 'db':
-                result = self.read_db_source(source)
-            elif ((generate is not None) and
-                  (generate in handlers) and
-                  (callable(handlers[generate]))):
-                result = handlers[generate](source)
-            else:
-                raise Exception(f"Invalid specification: {name}")
+            try:
+                if nature == 'db':
+                    result = self.read_db_source(source)
+                elif ((generate is not None) and
+                      (generate in handlers) and
+                      (callable(handlers[generate]))):
+                    result = handlers[generate](source)
+                elif ((generate is not None) and
+                      (hasattr(self, generate))):
+                    result = getattr(self, generate)(source)
+                else:
+                    raise Exception(f"Invalid specification: {name}")
+            except:
+                logger.exception(f"[{name}] generation failed",
+                                 extra={
+                                     'transform': self.name
+                                 })
 
             # Clean the read the dataset...
-            if pipeline is not None:
-                for processor in pipeline:
-                    if isinstance(processor, str):
-                        if processor in handlers:
-                            result = handlers[processor](result, source)
+            try:
+                if pipeline is not None and isinstance(pipeline, list):
+                    for processor in pipeline:
+                        if isinstance(processor, str):
+                            if processor in handlers:
+                                result = handlers[processor](result, source)
+                            elif hasattr(self, processor):
+                                result = getattr(self, processor)(result, source)
+                            else:
+                                raise Exception(f"Missing post-processor: {processor}")
+                        elif callable(processor):
+                            result = processor(result, source)
                         else:
-                            raise Exception(f"Missing post-processor: {processor}")
-                    elif callable(processor):
-                        result = processor(result, source)
-                    else:
-                        raise Exception("Only method names/callables are supported are supported")
-
+                            raise Exception("Only method names/callables are supported are supported")
+            except:
+                logger.exception(f"[{name}] post-processing failed",
+                                 extra={
+                                     'transform': self.name
+                                 })
             # We could return multiple values or a single value
             if isinstance(result, dict):
                 datasets.update(result)
             else:
                 datasets[name] = df
 
+
         return datasets
 
     def get_spec_sources(self, spec, datasets):
@@ -285,6 +368,8 @@ class MetricsBase(Compute):
                 if isinstance(processor, str):
                     if processor in handlers:
                         data = handlers[processor](data, spec)
+                    elif hasattr(self, processor):
+                        data = getattr(self, processor)(data, spec)
                     else:
                             raise Exception(f"Missing cleaner: {processor}")
                 elif callable(processor):
@@ -301,23 +386,24 @@ class MetricsBase(Compute):
 
         # Custom...
         generate = spec['generate']
-        if ((generate not in handlers) or
-            (not callable(handlers[generate]))):
+        callback = None
+        if ((generate in handlers) and (not callable(handlers[generate]))):
+            callback = handlers[generate]
+        elif hasattr(self, generate):
+            callback = getattr(self, generate)
+
+        if callback is None:
             raise Exception(f"[{name}] Invalid callback: {generate}")
 
         # Get hold of the data first...
         sources = self.get_spec_sources(spec, datasets)
 
-        # Call the custom handler that will generate the result for us
-        callback = handlers[generate]
-
         return callback(sources, spec)
 
     def process_spec_default(self, datasets, profile, spec):
         """
         Handle one specification at a time..
         """
-        handlers = self.get_handlers(profile)
 
         if (('dimensions' not in spec) or
             (not isinstance(spec['dimensions'], dict))):
@@ -384,6 +470,82 @@ class MetricsBase(Compute):
             spec['name']: df
         }
 
+    def get_dataset_generic(self, source):
+        """
+        Use the dataset object to read the dataset
+        """
+
+        if ((not hasattr(self, 'read_data')) or
+            (not hasattr(self, 'get_dataset'))):
+            raise Exception(" get_dataset_raw expects read_data and get_dataset methods")
+
+        args = self.args
+        start_date = self.args['start_date']
+        end_date = self.args['end_date']
+
+        name = source['name']
+        dataset = source['dataset']
+        params = source.get('params', {})
+        setdate = source.get('date', False)
+
+        cache = args.get('cache', False)
+        cachename= f"{dataset}-{start_date}-{end_date}"
+        cachefile = f"cache/{self.name}-cache-" + cachename + ".csv"
+
+        if cache:
+            try:
+                os.makedirs(os.path.dirname(cachefile))
+            except:
+                pass
+            if os.path.exists(cachefile):
+                logger.debug("Read cached {}".format(name),
+                             extra={
+                                 'transform': self.name
+                             })
+
+                df = pd.read_csv(cachefile, **params)
+                return {
+                    name: df
+                }
+
+        datasetobj = self.get_dataset(dataset)
+        df, metadata = datasetobj.read_data(start_date,
+                                            end_date,
+                                            filename=source['filename'],
+                                            readfunc=self.read_data,
+                                            params=params
+        )
+
+        logger.debug("Read {}".format(name),
+                     extra={
+                         'transform': self.name
+                     })
+
+        # Cache it for future use...
+        if cache:
+            df.to_csv(cachefile, index=False)
+
+        # Insert lineage if possible
+        lineage = None
+        if (('files' in metadata) and (len(metadata['files']) > 0)):
+            lineage = {
+                "type": "lineage",
+                "transform": self.name,
+                "dependencies": [
+                    {
+                        "type": "file",
+                        "nature": "input",
+                        "objects": [metadata['files'][-1]]
+                    },
+                ]
+            }
+
+        self.update_frame(source, df, lineage)
+
+        return {
+            name: df
+        }
+
     def process(self, state):
         """
         Run the computation and update the state
@@ -398,12 +560,12 @@ class MetricsBase(Compute):
 
         profile = self.get_profile()
 
-        # First get the datasets
-        datasets = self.get_datasets(profile)
-
         # Get specs..
         specs = self.get_specs(profile)
 
+        # First get the datasets
+        datasets = self.get_datasets(profile, specs)
+
         # Now go through each spec and get the output..
         for spec in specs:
             enable = spec.get('enable', True)
