@@ -7,6 +7,7 @@ import json
 import copy
 import shutil
 import logging
+import tempfile
 from inspect import signature
 import multiprocessing as mp
 import numpy as np
@@ -18,6 +19,7 @@ import s3fs
 import gcsfs
 
 from ..lib.misc import get_checksum, get_file_size
+from ..lib.integration import send_html_email
 
 from .frames import PandasDataFrame
 
@@ -26,6 +28,7 @@ logger = logging.getLogger('app')
 __all__ = ['S3Mixin', 'GCSMixin', 'PandasMixin',
            'ParallelMixin', 'AWSMixin',
            'FilesMixin', 'CheckpointMixin',
+           'EmailMixin'
        ]
 
 class CloudMixin(object):
@@ -235,8 +238,10 @@ class S3Mixin(CloudMixin):
             raise Exception("Could not find the credentials")
 
         return s3fs.S3FileSystem(
+            anon=False,
             key    = cred['access_key'],
             secret = cred['secret_key'],
+            use_listings_cache=False,
             client_kwargs=client_kwargs
         )
 
@@ -1267,3 +1272,58 @@ class CheckpointMixin(object):
         # Done
         return metadata
 
+class EmailMixin(object):
+    """
+    Send email
+    """
+
+    def send_email_helper(self, 
+                          sender,
+                          receivers,
+                          subject,
+                          body,
+                          attachment_prefix="",
+                          attachment_suffix="",
+                          frames={}):
+        """
+        Send email to a destination..
+        """
+
+        assert isinstance(frames, dict)
+
+        try:
+            tmpdir = tempfile.mkdtemp()
+
+            filenames = []
+            for name, df in frames.items():
+                filename = os.path.join(tmpdir, f'{attachment_prefix}{name}{attachment_suffix}.csv')
+                filenames.append(filename)
+                df.to_csv(filename, index=False)
+
+            send_html_email(body,
+                            sender=sender,
+                            receivers=receivers,
+                            subject=subject,
+                            attachments=filenames)
+
+            logger.debug("Email notification sent",
+                         extra={
+                             'transform': self.name,
+                             'data': "Receivers: " + json.dumps(receivers)
+                         })
+        except:
+            logger.exception("Unable to send email",
+                         extra={
+                             'transform': self.name,
+                         })
+
+        if os.path.exists(tmpdir):
+            try:
+                shutil.rmtree(tmpdir)
+            except:
+                logger.exception("Unable to cleanup temporary directory",
+                                 extra={
+                                     'transform': self.name,
+                                     'data': tmpdir
+                                 })
+    
