@@ -12,12 +12,14 @@ import markdown
 import importlib
 import traceback
 import s3fs
+import pandas as pd
 from datetime import datetime
 import abc
 import jsonschema
 
 from ..lib.exceptions import *
 from ..lib import renderlib
+from ..lib.misc import get_checksum, get_file_size
 
 logger = logging.getLogger('app')
 thisdir = os.path.dirname(__file__)
@@ -189,7 +191,8 @@ class Node(metaclass=NodeMeta):
         """
 
         self.args_schema = {}
-        """JSON Schema to validate the input args
+        """
+        dict: JSON Schema to validate the input args::
 
            See: https://python-jsonschema.readthedocs.io/en/latest/
                 https://json-schema.org/
@@ -214,8 +217,9 @@ class Node(metaclass=NodeMeta):
         """
 
         self.supported_extra_args = []
-        """Extra arguments that can be passed on the commandline to
-        the module.
+        """
+        Extra arguments that can be passed on the command line and the GUI to
+        the module::
 
             self.supported_extra_args = [
                 {
@@ -230,8 +234,11 @@ class Node(metaclass=NodeMeta):
             ]
 
         """
-        data_root = self.get_file("%(data_root)s")
 
+        self.data_root = self.get_file("%(data_root)s")
+        """
+
+        """
         self.testdata = self.get_testdata
         """dict: Test data to be used for testing module
 
@@ -361,7 +368,9 @@ class Node(metaclass=NodeMeta):
         """
         dict: Versions of the preprocessed data
 
-        .. deprecated:: 2.5
+        .. warning::
+
+            .. deprecated:: 2.0
 
         It has the form::
 
@@ -377,7 +386,10 @@ class Node(metaclass=NodeMeta):
 
         This attribute was used to sequence the transforms in early versions of Enrich. Now it is deprecated.
 
-        ..deprecated:: 2.0
+        .. warning::
+
+            .. deprecated:: 2.0
+
 
         """
 
@@ -387,7 +399,9 @@ class Node(metaclass=NodeMeta):
 
         This attribute was used to sequence the transforms in early versions of Enrich. Now it is deprecated.
 
-        ..deprecated:: 2.0
+        .. warning::
+
+           .. deprecated:: 2.0
 
         """
 
@@ -554,8 +568,11 @@ class Node(metaclass=NodeMeta):
         -------
         bool
 
-        .. deprecated:: 2.0.10
-             Use enrichsdk.render library functions
+        .. warning::
+
+           .. deprecated:: 2.0
+           Use enrichsdk.render library functions
+
         """
         return self.node_type == "Skin"
 
@@ -567,8 +584,10 @@ class Node(metaclass=NodeMeta):
         -------
         bool
 
-        .. deprecated:: 2.0.10
-             Not used. Will be dropped in future
+        .. warning::
+
+           .. deprecated:: 2.0
+           Will be dropped in future
 
         """
         return self.node_type == "Integration"
@@ -635,12 +654,11 @@ class Node(metaclass=NodeMeta):
 
     def get_supported_extra_args(self):
         """
-        What extra command line arguments are supported
+        Get the command line arguments. This will handle any callables specified
+        by the default.
 
-        Returns
-        -------
-        list: List of arg specifications. Default values can be
-              actual strings/numbers or callback functions
+        Returns:
+           args (list): List of arg specifications. Default values can be actual   strings/numbers or callback functions
 
         See supported_extra_args
         """
@@ -661,9 +679,12 @@ class Node(metaclass=NodeMeta):
         """
         Clean the arguments before using them
 
-        Parameters
-        ----------
-        args: Args specified in the config file
+        Args:
+           args: args to be resolved/cleaned/extended
+
+        Returns:
+           list: Cleaned args
+
         """
 
         # Nothing to clean if there are no extra args that could be
@@ -765,9 +786,12 @@ class Node(metaclass=NodeMeta):
         """
         Check whether this configuration is even valid?
 
-        Parameters
-        ----------
-        conf: Module configuration specified in the config file
+        Args:
+           conf: Module configuration specified in the config file
+
+        Raises:
+            Generic exception if conf is not a dictionary or if the version doesnt match
+
         """
 
         if not (isinstance(conf, dict) and len(conf) > 0):
@@ -835,9 +859,8 @@ class Node(metaclass=NodeMeta):
         """
         Load the module and prepare to execute
 
-        Parameters
-        ----------
-        conf - Module configuration specified in config
+        Args:
+           conf: Module configuration specified in config
 
         """
         readonly = self.config.readonly
@@ -1075,11 +1098,8 @@ class Node(metaclass=NodeMeta):
         This can be used to open connections, setup state and other
         functions.
 
-        Parameters
-        ----------
-        None
-
         .. deprecated::
+
            Use configure/other functions to complete the initialization..
 
         """
@@ -1127,9 +1147,9 @@ class Node(metaclass=NodeMeta):
 
         Every module implements this function except Skins.
 
-        Parameters
-        ----------
-        state: State object passed by the pipeline manager
+        Args:
+           state: State object passed by the pipeline manager
+
         """
         pass
 
@@ -1143,6 +1163,8 @@ class Node(metaclass=NodeMeta):
 
         This name should in specific in the credentials
         section of siteconf.
+
+
         """
         if not hasattr(self.config, 'siteconf'):
             raise Exception("No siteconf")
@@ -1200,6 +1222,10 @@ class Node(metaclass=NodeMeta):
 
         usecase = self.config.get_usecase()
 
+        versionmap = self.get_versionmap()
+        for v in versionmap:
+            v.pop('branches',None)
+
         metadata = {
             "schema": "standalone:pipeline",
             "version": "1.0",
@@ -1214,7 +1240,7 @@ class Node(metaclass=NodeMeta):
                 "pid":  state.pid,
                 "start_time":   state.start_time.replace(microsecond=0).isoformat(),
                 "end_time":   state.end_time.replace(microsecond=0).isoformat(),
-                "versionmap": self.get_versionmap(),
+                "versionmap": versionmap,
                 "stats":   state.stats,
                 "cmdline":  list(sys.argv),
             }
@@ -1255,6 +1281,11 @@ class Node(metaclass=NodeMeta):
     def get_relative_path(self, path, what="enrich_data_dir"):
         """
         Get the path relative to a predefined root
+
+        Args:
+             path (str): Path of object
+             what (str): What should the root of the relative path be?
+
         """
         return self.config.get_relative_path(path, what)
 
@@ -1264,6 +1295,7 @@ class Node(metaclass=NodeMeta):
 
         Args:
              filename (str): File being accessed
+             args (list): Extra args to be passed to baseclass for use
         """
         self.config.note_access(filename, self, *args, **kwargs)
 
@@ -1274,10 +1306,12 @@ class Node(metaclass=NodeMeta):
                       create=False,
                       extra={}):
         """
-        ..deprecated:: 2.5
-
         Get a local directory for caching partial results.
 
+        .. warning::
+
+            Deprecated. Not to be used.
+
         Args:
              name (str): Namespace
              version (str): Version of the cache
@@ -1286,7 +1320,7 @@ class Node(metaclass=NodeMeta):
              extra (dict): Extra parameters passed to get_file
 
         Returns:
-             path: Path of the cache directory
+             path (str): Path of the cache directory
 
         """
         warnings.warn("Function will be removed soon", DeprecationWarning)
@@ -1439,6 +1473,58 @@ class Transform(Node):
     #####################################################
     #=> Generate state management information
     #####################################################
+    def get_file_params(self, name, df, path, fshandle=None):
+        """
+        Generate output metadata for files (required for compliance)
+
+        Args:
+            df (Dataframe): Pandas dataframe
+            path (str): Path where the dataframe was dumped
+            fshandle (obj): S3 filesystem handle of (s3fs)
+
+        Returns:
+            list: List of dicts
+
+        """
+
+        # Collect file information for files that are written to s3/filesystem
+        if fshandle is not None:
+            info = fshandle.info(path)
+            component = {
+                'filename': path,
+                'sha256sum': info['ETag'].replace('"',''),
+                "filesize": "{0:0.3f} MB".format(info['size']/(1024*1024)),
+                "modified_time": info['LastModified'].ctime(),
+                "create_time": info['LastModified'].ctime(),
+            }
+        else:
+            root = self.config.enrich_data_dir
+            component = {
+                'filename': os.path.relpath(path, root),
+                'sha256sum': get_checksum(path),
+                'filesize': "{0:0.3f} MB".format(get_pathsize(path)/(1024*1024)),
+                "modified_time": str(time.ctime(os.path.getmtime(path))),
+                "create_time": str(time.ctime(os.path.getctime(path))),
+            }
+
+        # Insert shape as well..
+        component.update({
+            "rows": df.shape[0],
+            "columns": df.shape[1]
+        })
+
+        return [
+            {
+                "action": "output",
+                "type": "table",
+                "frametype": "pandas",
+                "filename": path,
+                "components": [component],
+                "columns": self.get_column_metadata(name, df),
+                'ddl': pd.io.sql.get_schema(df, name)
+            }
+        ]
+
     def get_column_params(self, name, df):
         """
         Generate columns metadata to be associated with
@@ -1471,10 +1557,14 @@ class Transform(Node):
         Returns:
             Dict with columns metadata
         """
+
+        sampledf = df.sample(min(3, df.shape[0]))
+
         columns = {}
-        for c in list(df.columns):
-            sample = list(df[c].sample(min(3, df.shape[0])))
-            sample = [flatten(c) for c in sample]
+        for index, c in enumerate(df.columns):
+
+            # Handle the case where there are duplicate column names
+            sample = [flatten(c) for c in sampledf.iloc[:,index].tolist()]
             columns[c] = {
                 'touch': self.name, # Who is introducing this column
                 'datatype': df.get_generic_dtype(c), # What is its type
