@@ -79,7 +79,7 @@ class EnrichStateBase(object):
             'user': user
         }
 
-    def update_frame(self, name, detail, create=False):
+    def update_frame(self, name, detail, create=True):
         """
         Update the state with the details of a dataframe
 
@@ -88,7 +88,6 @@ class EnrichStateBase(object):
             detail (dict): Dict with a number of details 
             create (bool): Add if the frame doesnt exist.
 
-
         The detail has the elements mentioned above::
 
             {
