@@ -2,6 +2,7 @@ import os
 import sys
 import json
 import re
+from datetime import datetime
 import traceback
 import pandas as pd
 from s3fs import S3FileSystem
@@ -659,9 +660,25 @@ class Dataset(DataSource):
                   errors=True,
                   name="default",
                   params={},
-                  resolve={}):
+                  resolve={}
+    ):
         """
         Read a single dataset
+
+        Args:
+             start_date (str): Starting date to scan
+             end_date (str): Ending date of the scane
+             filename (str): Filename within each day's directory
+             readfunc (method): Callback 
+             errors (bool): What to do on failure. True = bailout
+             name (str): Which subset to look at?
+             params (dict): Params to the readfunc
+             resolve (dict): Parameters for resolution of path
+
+        Returns:
+             dataframe: Data read 
+             dict: Optional metadata on the read
+
         """
 
         dates = self.generate(start_date, end_date)
@@ -671,11 +688,16 @@ class Dataset(DataSource):
         # Get the path template...
         path = self.get_path_by_name(name, resolve)
 
+        metadata = {
+            'start': datetime.now(),
+            'files': []
+        }
         # Now compose all the dataframes...
         _dfs = []
         for dt in dates:
             fullpath = os.path.join(path, dt['name'], filename)
-            try: 
+            try:
+                metadata['files'].append(fullpath)
                 df = readfunc(fullpath, params, date=dt['name'])
                 if ((df is None) or (len(df) == 0)):
                     continue
@@ -687,7 +709,10 @@ class Dataset(DataSource):
         df = pd.concat(_dfs, ignore_index=True)
         del _dfs
 
-        return df
+        metadata['end'] = datetime.now()
+
+        return df, metadata
+
 
 class DatabaseTable(DataSource):
 
