@@ -247,7 +247,9 @@ def _start(commands):
                                  'transform-package',
                                  'transform-simple',
                                  'transform-helloworld',
+                                 'transform-query',
                                  'asset',
+                                 'dashboard',
                                  'task-lib',
                                  'task-conf',
                                  'pipeline-conf',
