@@ -131,8 +131,11 @@ class CloudMixin(object):
         env = dict(os.environ)
         if 'aws' in args:
             cred = self.get_credentials(args['aws']['credentials'])
-            env['AWS_ACCESS_KEY_ID'] = cred['access_key']
-            env['AWS_SECRET_ACCESS_KEY'] = cred['secret_key']
+            if cred is not None:
+                if (('access_key' in cred) and (isinstance(cred['access_key'], str))):
+                    env['AWS_ACCESS_KEY_ID'] = cred['access_key']
+                if (('secret_key' in cred) and (isinstance(cred['secret_key'], str))):
+                    env['AWS_SECRET_ACCESS_KEY'] = cred['secret_key']
         else:
             env['BOTO_CONFIG'] = self.config.get_file(args['gcp']['boto'])
 
